from datetime import datetime
from typing import Annotated, Optional

from litestar.params import Parameter
from msgspec import Struct

from stofory_sdk.catalog.models.enums import ProductType, ProductKind

from .product_platform_schema import ProductPlatformResponse


class ProductCreateRequest(Struct, forbid_unknown_fields=True):
    name: Annotated[
        str,
        Parameter(
            title="Name",
            description="The name of the product."
        )
    ]
    description: Annotated[
        str,
        Parameter(
            title="Description",
            description="The description of the product."
        )
    ]
    product_type: Annotated[
        ProductType,
        Parameter(
            title="Product Type",
            description="The type of the product."
        )
    ]
    product_kind: Annotated[
        ProductKind,
        Parameter(
            title="Product Kind",
            description="The kind of the product."
        )
    ]
    slug: Annotated[
        str,
        Parameter(
            title="Slug",
            description="The slug of the product."
        )
    ]
    origin_platform_id: Annotated[
        int,
        Parameter(
            title="Origin Platform ID",
            description="The ID of the origin platform."
        )
    ]
    region_buy_id: Annotated[
        int,
        Parameter(
            title="Region Buy ID",
            description="The ID of the region where the product is bought."
        )
    ]
    regions_sale_ids: Annotated[
        Optional[list[int]],
        Parameter(
            title="Regions Sale IDs",
            description="The list of IDs of regions where the product is sold."
        )
    ]


class ProductUpdateRequest(Struct, forbid_unknown_fields=True):
    name: Annotated[
        Optional[str],
        Parameter(
            title="Name",
            description="The name of the product."
        )
    ]
    description: Annotated[
        Optional[str],
        Parameter(
            title="Description",
            description="The description of the product."
        )
    ]
    product_type: Annotated[
        Optional[ProductType],
        Parameter(
            title="Product Type",
            description="The type of the product."
        )
    ]
    product_kind: Annotated[
        Optional[ProductKind],
        Parameter(
            title="Product Kind",
            description="The kind of the product."
        )
    ]
    status: Annotated[
        Optional[bool],
        Parameter(
            title="Status",
            description="The status of the product. True for active, False for inactive."
        )
    ]
    slug: Annotated[
        Optional[str],
        Parameter(
            title="Slug",
            description="The slug of the product."
        )
    ]
    origin_platform_id: Annotated[
        Optional[int],
        Parameter(
            title="Origin Platform ID",
            description="The ID of the origin platform."
        )
    ]
    region_buy_id: Annotated[
        Optional[int],
        Parameter(
            title="Region Buy ID",
            description="The ID of the region where the product is bought."
        )
    ]
    regions_sale_ids: Annotated[
        Optional[list[int]],
        Parameter(
            title="Regions Sale IDs",
            description="The list of IDs of regions where the product is sold."
        )
    ]


class ProductResponse(Struct):
    id: int
    name: str
    description: str
    product_type: ProductType
    product_kind: ProductKind
    status: bool
    slug: str
    origin_platform_id: int
    region_buy_id: int
    regions_sale_ids: Optional[list[int]]
    created_at: datetime
    updated_at: datetime

    product_platforms: list[ProductPlatformResponse]


class PlatformShortResponse(Struct):
    id: int
    name: str


class ProductPlatformShortWithPlatformResponse(Struct):
    id: int
    platform: PlatformShortResponse


class ProductShortenedResponse(ProductResponse):
    product_platforms: list[ProductPlatformShortWithPlatformResponse]



from .schemas import (
    ProductParametersResponse,
    ProductParameterResponse,
    ProductParameterCreateRequest,
    ProductParameterCreateResponse,
    ProductParameterUpdateRequest,
    ProductParameterUpdateResponse,
    ProductParameterDeleteResponse,
    ProductParameterVariantCreateRequest,
    ProductParameterVariantCreateResponse,
    ProductParameterVariantUpdateRequest,
    ProductParameterVariantUpdateResponse,
    ProductParameterVariantDeleteResponse,
)


__all__ = (
    "ProductParametersResponse",
    "ProductParameterResponse",
    "ProductParameterCreateRequest",
    "ProductParameterCreateResponse",
    "ProductParameterUpdateRequest",
    "ProductParameterUpdateResponse",
    "ProductParameterDeleteResponse",
    "ProductParameterVariantCreateRequest",
    "ProductParameterVariantCreateResponse",
    "ProductParameterVariantUpdateRequest",
    "ProductParameterVariantUpdateResponse",
    "ProductParameterVariantDeleteResponse",
)



from __future__ import annotations

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import Enum
from sqlalchemy.dialects.postgresql import VARCHAR, ARRAY, ENUM
from sqlalchemy.orm import Mapped, mapped_column

from .enums import SyncTaskType, SyncTaskPriority


class SyncQueueMapping(BigIntAuditBase):
    priority: Mapped[SyncTaskPriority] = mapped_column(ENUM(SyncTaskPriority, name="sync_task_priority_enum"), nullable=False)
    consumer: Mapped[str] = mapped_column(VARCHAR(255), nullable=False)
    table_names: Mapped[list[str]] = mapped_column(ARRAY(VARCHAR(255)), nullable=False)
    operation: Mapped[SyncTaskType] = mapped_column(ENUM(SyncTaskType, name="sync_task_type_enum"), nullable=False)



from typing import List

import msgspec
from stofory_sdk.catalog.models.enums import Locale, ProductKind, ProductType

__all__ = ('LanguagesInfoResponse', 'PlatformInfoResponse', 'CatalogFilterResponse')


class LanguagesInfoResponse(msgspec.Struct, forbid_unknown_fields=True):
    id: int
    language: Locale
    # is_enabled: bool


class PlatformInfoResponse(msgspec.Struct, forbid_unknown_fields=True):
    id: int
    name: str


class RegionInfo(msgspec.Struct, forbid_unknown_fields=True):
    id: int
    name_ru: str
    name_en: str
    alpha2: str


class OriginPlatformInfo(msgspec.Struct, forbid_unknown_fields=True):
    id: int
    name: str


class CatalogFilterResponse(msgspec.Struct, forbid_unknown_fields=True):
    regions: List[RegionInfo]
    origin_platforms: List[OriginPlatformInfo]
    product_types: List[ProductType]
    product_kinds: List[ProductKind]



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Index
from sqlalchemy.dialects.postgresql import BIGINT, INTEGER, BOOLEAN
from sqlalchemy.orm import Mapped, relationship, mapped_column

if TYPE_CHECKING:
    from .product_platform import ProductPlatform


class ProductPlatformDiscount(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_discount_product_platform_id", "product_platform_id"),
    )

    product_platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"),
                                                     nullable=False, unique=True)
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    unit_for_discount: Mapped[int] = mapped_column(INTEGER, nullable=False)
    discount: Mapped[int] = mapped_column(INTEGER, nullable=False)
    is_enabled: Mapped[bool] = mapped_column(BOOLEAN, default=False)

    product_platform: Mapped[ProductPlatform] = relationship(
        back_populates="discount",
        foreign_keys="ProductPlatformDiscount.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="joined"
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase, SlugKey
from sqlalchemy import Index, UniqueConstraint, ForeignKey
from sqlalchemy.dialects.postgresql import VARCHAR, ENUM, TEXT, BOOLEAN, BIGINT, ARRAY
from sqlalchemy.orm import Mapped, mapped_column, relationship, declared_attr

from .enums import ProductType
from .enums.product_kind import ProductKind


if TYPE_CHECKING:
    from .region import Region
    from .origin_platform import OriginPlatform
    from .product_platform import ProductPlatform
    from .product_platform_external_connection import ProductPlatformExternalConnection


class Product(BigIntAuditBase, SlugKey):
    @declared_attr.directive
    @classmethod
    def __table_args__(cls):
        return (
            Index("idx_product_name", "name"),
            UniqueConstraint(
                cls.slug,
                name=f"uq_{cls.__tablename__}_slug",
            ).ddl_if(callable_=cls._create_unique_slug_constraint),
            Index(
                f"ix_{cls.__tablename__}_slug_unique",
                cls.slug,
                unique=True,
            ).ddl_if(callable_=cls._create_unique_slug_index),
        )

    # __table_args__ = (
    #     Index("idx_product_name", "name"),
    # )

    name: Mapped[str] = mapped_column(VARCHAR(255), nullable=False)
    description: Mapped[str] = mapped_column(TEXT(), nullable=False)
    product_type: Mapped[ProductType] = mapped_column(ENUM(ProductType, name='product_type_enum'), nullable=False)
    product_kind: Mapped[ProductKind] = mapped_column(ENUM(ProductKind, name='product_kind_enum'), nullable=False)
    status: Mapped[bool] = mapped_column(BOOLEAN(), default=False)

    origin_platform_id: Mapped[int] = mapped_column(
        BIGINT,
        ForeignKey(
            "origin_platform.id",
            ondelete="RESTRICT"
        ),
        nullable=False
    )
    region_buy_id: Mapped[int] = mapped_column(
        BIGINT,
        ForeignKey(
            "region.id",
            ondelete="RESTRICT"
        ),
        nullable=False
    )
    regions_sale_ids: Mapped[list[int]] = mapped_column(
        ARRAY(BIGINT),
        nullable=True
    )

    origin_platform: Mapped[OriginPlatform] = relationship(
        back_populates="products",
        foreign_keys="Product.origin_platform_id",
        lazy="joined",
        uselist=True,
        passive_deletes=True,
    )

    region_buy: Mapped[Region] = relationship(
        lazy="joined",
        back_populates="product_bought",
    )

    regions_sale: Mapped[list[Region]] = relationship(
        primaryjoin="Product.regions_sale_ids.any(foreign(Region.id))",
        viewonly=True,
        lazy="noload",
        # back_populates="products_sold",
    )

    product_platforms: Mapped[list[ProductPlatform]] = relationship(
        back_populates="product",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan"
    )

    external_connections: Mapped[list[ProductPlatformExternalConnection]] = relationship(
        back_populates="product",
        lazy="noload",
        uselist=True,
        # cascade="all, delete-orphan"
    )


"""
CREATE OR REPLACE FUNCTION check_regions_sale_ids_exist()
RETURNS TRIGGER AS $$
BEGIN
    -- Проверяем, есть ли несуществующие region_id в regions_sale_ids
    IF EXISTS (
        SELECT 1
        FROM unnest(NEW.regions_sale_ids) AS region_id
        WHERE NOT EXISTS (SELECT 1 FROM region WHERE id = region_id)
    ) THEN
        RAISE EXCEPTION 'One or more regions in regions_sale_ids do not exist in the region table';
    END IF;

    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

-- Привязываем функцию к таблице product
CREATE TRIGGER trg_check_regions_sale_ids
BEFORE INSERT OR UPDATE OF regions_sale_ids ON product
FOR EACH ROW
EXECUTE FUNCTION check_regions_sale_ids_exist();

"""


import enum


class ProductKind(enum.StrEnum):
    ACCOUNT = enum.auto()  # "Аккаунты (Accounts)"
    KEY = enum.auto()  # "Ключи (Keys)"
    OTHER = enum.auto()  # "Другое (Others)"
    SERVICE = enum.auto()  # "Услуги (Services)"



from typing import Any, Dict

import msgspec.json
from sqlalchemy.dialects.postgresql import JSONB

from sqlalchemy.types import TypeDecorator
import base64


class JSONBDict(TypeDecorator):
    impl = JSONB

    def process_result_value(self, value: Any, dialect: Any) -> Dict[str, Any]:
        if value is None:
            return {}
        if isinstance(value, str):
            try:
                return msgspec.json.decode(value)
            except msgspec.DecodeError:
                try:
                    decoded = base64.b64decode(value).decode('utf-8')
                    return msgspec.json.decode(decoded)
                except Exception:
                    return {"value": value}
        return value



from .product import Product
from .platform import Platform
from .category import Category
from .category_name import CategoryName
from .localization import Localization

from .origin_platform import OriginPlatform

from .product import Product
from .product_platform_image import ProductPlatformImage
from .product_platform_parameter import ProductPlatformParameter
from .product_platform_pricing import ProductPlatformPricing
from .product_platform_video import ProductPlatformVideo
from .product_platform import ProductPlatform

from .parameter_name import ParameterName
from .parameter_comment import ParameterComment
from .parameter_option import ParameterOption

from .parameter_option_name import ParameterOptionName

from .product_platform_name import ProductPlatformName
from .product_platform_description import ProductPlatformDescription
from .product_platform_category import ProductPlatformCategory
from .product_platform_bonus import ProductPlatformBonus
from .product_platform_discount import ProductPlatformDiscount
from .product_platform_guarantee import ProductPlatformGuarantee

from .product_platform_sync_task import ProductPlatformSyncTask
from .product_platform_external_connection import ProductPlatformExternalConnection
from .product_platform_versioning import ProductPlatformVersioning

from .region import Region

from .sync_queue_mapping import SyncQueueMapping

from .origin_platform import OriginPlatform
from .region import Region

from .product_platform_content import ProductPlatformContent


__all__ = (
    "OriginPlatform",

    "Product",
    "Platform",
    "Category",
    "CategoryName",
    "Localization",

    "ProductPlatformImage",
    "ProductPlatformParameter",
    "ProductPlatformPricing",
    "ProductPlatformVideo",
    "ProductPlatform",

    "ParameterName",
    "ParameterComment",
    "ParameterOption",

    "ParameterOptionName",

    "ProductPlatformName",
    "ProductPlatformDescription",
    "ProductPlatformCategory",
    "ProductPlatformBonus",
    "ProductPlatformDiscount",
    "ProductPlatformGuarantee",

    "ProductPlatformSyncTask",
    "ProductPlatformExternalConnection",
    "ProductPlatformVersioning",

    "SyncQueueMapping",

    "OriginPlatform",
    "Region",

    "ProductPlatformContent",
)



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct

from stofory_sdk.catalog.models.enums import Currency, PricingType


class ProductPlatformPricingCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID")]
    pricing_type: Annotated[
        PricingType,
        Parameter(
            title="Pricing Type",
            description="The pricing type."
        )
    ]
    currency: Annotated[
        Currency,
        Parameter(
            title="Currency",
            description="The currency."
        )
    ]
    price: Annotated[
        int | None,
        Parameter(
            required=False,
            title="Price",
            description="The price."
        )
    ] = None
    price_per_unit: Annotated[
        int | None,
        Parameter(
            required=False,
            title="Price Per Unit",
            description="The price per unit."
        )
    ] = None
    min_quantity: Annotated[
        int | None,
        Parameter(
            required=False,
            title="Min Quantity",
            description="The min quantity."
        )
    ] = None
    max_quantity: Annotated[
        int | None,
        Parameter(
            required=False,
            title="Max Quantity",
            description="The max quantity."
        )
    ] = None
    unit_name: Annotated[
        str | None,
        Parameter(
            required=False,
            title="Unit Name",
            description="The unit name."
        )
    ] = None


class ProductPlatformPricingUpdateRequest(Struct, forbid_unknown_fields=True):
    pricing_type: Annotated[
        PricingType,
        Parameter(
            title="Pricing Type",
            description="The pricing type."
        )
    ]
    currency: Annotated[
        Currency,
        Parameter(
            title="Currency",
            description="The currency."
        )
    ]
    price: Annotated[
        int | None,
        Parameter(
            required=False,
            title="Price",
            description="The price."
        )
    ] = None
    price_per_unit: Annotated[
        int | None,
        Parameter(
            required=False,
            title="Price Per Unit",
            description="The price per unit."
        )
    ] = None
    min_quantity: Annotated[
        int | None,
        Parameter(
            required=False,
            title="Min Quantity",
            description="The min quantity."
        )
    ] = None
    max_quantity: Annotated[
        int | None,
        Parameter(
            required=False,
            title="Max Quantity",
            description="The max quantity."
        )
    ] = None
    unit_name: Annotated[
        str | None,
        Parameter(
            required=False,
            title="Unit Name",
            description="The unit name."
        )
    ] = None


class ProductPlatformPricingResponse(Struct):
    id: int
    product_platform_id: int
    version: int
    pricing_type: PricingType
    currency: Currency
    price: int | None
    price_per_unit: int | None
    min_quantity: int | None
    max_quantity: int | None
    unit_name: str | None
    created_at: datetime
    updated_at: datetime



from .product_schema import ProductResponse, ProductUpdateRequest, ProductCreateRequest
from .category_schema import CategoryResponse, CategoryUpdateRequest, CategoryCreateRequest
from .localization_schema import LocalizationResponse, LocalizationUpdateRequest, LocalizationCreateRequest
from .platform_schema import PlatformResponse, PlatformUpdateRequest, PlatformCreateRequest

from .product_platform_schema import ProductPlatformResponse, ProductPlatformUpdateRequest, ProductPlatformCreateRequest
from .product_platform_parameter_schema import ProductPlatformParameterResponse, ProductPlatformParameterUpdateRequest, ProductPlatformParameterCreateRequest
from .product_platform_image_schema import ProductPlatformImageResponse, ProductPlatformImageUpdateRequest, ProductPlatformImageCreateRequest
from .product_platform_video_schema import ProductPlatformVideoResponse, ProductPlatformVideoUpdateRequest, ProductPlatformVideoCreateRequest

from .product_platform_category_schema import ProductPlatformCategoryResponse, ProductPlatformCategoryUpdateRequest, ProductPlatformCategoryCreateRequest
from .product_platform_name_schema import ProductPlatformNameResponse, ProductPlatformNameUpdateRequest, ProductPlatformNameCreateRequest
from .product_platform_description_schema import ProductPlatformDescriptionResponse, ProductPlatformDescriptionUpdateRequest, ProductPlatformDescriptionCreateRequest
from .product_platform_discount_schema import ProductPlatformDiscountResponse, ProductPlatformDiscountUpdateRequest, ProductPlatformDiscountCreateRequest
from .product_platform_bonus_schema import ProductPlatformBonusResponse, ProductPlatformBonusUpdateRequest, ProductPlatformBonusCreateRequest
from .product_platform_guarantee_schema import ProductPlatformGuaranteeResponse, ProductPlatformGuaranteeUpdateRequest, ProductPlatformGuaranteeCreateRequest
from .product_platform_pricing_schema import ProductPlatformPricingResponse, ProductPlatformPricingUpdateRequest, ProductPlatformPricingCreateRequest

from .parameter_name_schema import ParameterNameResponse, ParameterNameUpdateRequest, ParameterNameCreateRequest
from .parameter_comment_schema import ParameterCommentResponse, ParameterCommentUpdateRequest, ParameterCommentCreateRequest
from .parameter_option_schema import ParameterOptionResponse, ParameterOptionUpdateRequest, ParameterOptionCreateRequest
from .parameter_option_name_schema import ParameterOptionNameResponse, ParameterOptionNameUpdateRequest, ParameterOptionNameCreateRequest

from .product_platform_sync_task_schema import ProductPlatformSyncTaskResponse, ProductPlatformSyncTaskUpdateRequest, ProductPlatformSyncTaskCreateRequest
from .product_platform_versioning_schema import ProductPlatformVersioningResponse, ProductPlatformVersioningUpdateRequest, ProductPlatformVersioningCreateRequest
from .product_platform_external_connection_schema import ProductPlatformExternalConnectionResponse, ProductPlatformExternalConnectionUpdateRequest, ProductPlatformExternalConnectionCreateRequest

from .action_schema import ValidateProductPlatfromEnablingResponse, CheckProductPlatformSyncResponse, SyncProductPlatformResponse
from .enums_schema import EnumsListResponse, EnumListResponse



__all__ = (
    "ProductResponse",
    "ProductUpdateRequest",
    "ProductCreateRequest",

    "CategoryResponse",
    "CategoryUpdateRequest",
    "CategoryCreateRequest",

    "LocalizationResponse",
    "LocalizationUpdateRequest",
    "LocalizationCreateRequest",

    "PlatformResponse",
    "PlatformUpdateRequest",
    "PlatformCreateRequest",

    "ProductPlatformResponse",
    "ProductPlatformUpdateRequest",
    "ProductPlatformCreateRequest",

    "ProductPlatformParameterResponse",
    "ProductPlatformParameterUpdateRequest",
    "ProductPlatformParameterCreateRequest",

    "ProductPlatformImageResponse",
    "ProductPlatformImageUpdateRequest",
    "ProductPlatformImageCreateRequest",

    "ProductPlatformVideoResponse",
    "ProductPlatformVideoUpdateRequest",
    "ProductPlatformVideoCreateRequest",

    "ProductPlatformCategoryResponse",
    "ProductPlatformCategoryUpdateRequest",
    "ProductPlatformCategoryCreateRequest",

    "ProductPlatformNameResponse",
    "ProductPlatformNameUpdateRequest",
    "ProductPlatformNameCreateRequest",

    "ProductPlatformDescriptionResponse",
    "ProductPlatformDescriptionUpdateRequest",
    "ProductPlatformDescriptionCreateRequest",

    "ProductPlatformDiscountResponse",
    "ProductPlatformDiscountUpdateRequest",
    "ProductPlatformDiscountCreateRequest",

    "ProductPlatformBonusResponse",
    "ProductPlatformBonusUpdateRequest",
    "ProductPlatformBonusCreateRequest",

    "ProductPlatformGuaranteeResponse",
    "ProductPlatformGuaranteeUpdateRequest",
    "ProductPlatformGuaranteeCreateRequest",

    "ProductPlatformPricingResponse",
    "ProductPlatformPricingUpdateRequest",
    "ProductPlatformPricingCreateRequest",

    "ParameterNameResponse",
    "ParameterNameUpdateRequest",
    "ParameterNameCreateRequest",

    "ParameterCommentResponse",
    "ParameterCommentUpdateRequest",
    "ParameterCommentCreateRequest",

    "ParameterOptionResponse",
    "ParameterOptionUpdateRequest",
    "ParameterOptionCreateRequest",

    "ParameterOptionNameResponse",
    "ParameterOptionNameUpdateRequest",
    "ParameterOptionNameCreateRequest",

    "ProductPlatformSyncTaskResponse",
    "ProductPlatformSyncTaskUpdateRequest",
    "ProductPlatformSyncTaskCreateRequest",

    "ProductPlatformVersioningResponse",
    "ProductPlatformVersioningUpdateRequest",
    "ProductPlatformVersioningCreateRequest",

    "ProductPlatformExternalConnectionResponse",
    "ProductPlatformExternalConnectionUpdateRequest",
    "ProductPlatformExternalConnectionCreateRequest",

    "ValidateProductPlatfromEnablingResponse",
    "CheckProductPlatformSyncResponse",
    "SyncProductPlatformResponse",

    "EnumsListResponse",
    "EnumListResponse",


)



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import BigIntAuditBase
from sqlalchemy import ForeignKey, Enum, Index
from sqlalchemy.dialects.postgresql import BIGINT, BOOLEAN, INTEGER
from sqlalchemy.orm import Mapped, relationship, mapped_column

from .enums import ParameterType


if TYPE_CHECKING:
    from .product_platform import ProductPlatform
    from .parameter_name import ParameterName
    from .parameter_comment import ParameterComment
    from .parameter_option import ParameterOption


class ProductPlatformParameter(BigIntAuditBase):
    __table_args__ = (
        Index("idx_product_platform_parameter_product_platform_id", "product_platform_id"),
    )

    product_platform_id: Mapped[int] = mapped_column(BIGINT, ForeignKey("product_platform.id", ondelete="CASCADE"), nullable=False)
    version: Mapped[int] = mapped_column(BIGINT, default=1)
    parameter_type: Mapped[ParameterType] = mapped_column(Enum(ParameterType, name="parameter_type_enum"), nullable=False)
    is_required: Mapped[bool] = mapped_column(BOOLEAN, default=False)
    order: Mapped[int] = mapped_column(INTEGER, nullable=False)

    product_platform: Mapped[list[ProductPlatform]] = relationship(
        back_populates="parameters",
        foreign_keys="ProductPlatformParameter.product_platform_id",
        innerjoin=True,
        uselist=False,
        lazy="noload"   # select/noload?
    )

    names: Mapped[list[ParameterName]] = relationship(
        back_populates="parameter",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ParameterName.id)"
    )

    comments: Mapped[list[ParameterComment]] = relationship(
        back_populates="parameter",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ParameterComment.id)"
    )

    options: Mapped[list[ParameterOption]] = relationship(
        back_populates="parameter",
        lazy="noload",
        uselist=True,
        passive_deletes=True,
        cascade="all, delete-orphan",
        order_by="asc(ParameterOption.id)"
    )



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct

from ..models.enums import ContentType


class ProductPlatformContentCreateRequest(Struct):
    product_platform_id: Annotated[int, Parameter(int, ge=1, title="Product Platform ID")]
    content_type: Annotated[ContentType, Parameter(ContentType, title="Content Type", description="The type of content.")]
    value: Annotated[str, Parameter(str, title="Content", description="The content of the product platform.")]


class ProductPlatformContentUpdateRequest(Struct):
    value: Annotated[str, Parameter(str, title="Content", description="The content of the product platform.")]


class ProductPlatformContentResponse(Struct):
    id: int
    product_platform_id: int
    content_type: ContentType
    value: str
    created_at: datetime
    updated_at: datetime



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct

from .localization_schema import LocalizationResponse


class ProductPlatformNameCreateRequest(Struct, forbid_unknown_fields=True):
    product_platform_id: Annotated[int, Parameter(title="Product Platform ID", description="The ID of the product platform.")]
    localization_id: Annotated[int, Parameter(title="Localization ID", description="The ID of the localization.")]
    value: Annotated[str, Parameter(title="Value", description="The value.")]


class ProductPlatformNameUpdateRequest(Struct, forbid_unknown_fields=True):
    # localization_id: Annotated[
    #     int | None,
    #     Parameter(
    #         required=False,
    #         title="Localization ID",
    #         description="The ID of the localization."
    #     )
    # ] = None
    value: Annotated[
        str,
        Parameter(
            title="Value",
            description="The value."
        )
    ]


class ProductPlatformNameResponse(Struct):
    id: int
    product_platform_id: int
    localization_id: int
    version: int
    value: str
    created_at: datetime
    updated_at: datetime

    locale: LocalizationResponse



from .schemas import (
    ContentFileNode,
    AddTextNode,
    ContentTextNode,
    ContentCode,
    ContentFileEditNode,
    ContentDeleteAll,
    DeliveryTypes,
    ContentForm,
    AddTextRequest,
    EditTextRequest,
    CreateEditFormRequest,
    AddFileResponse,
    AddTextResponse,
    UpdateCodeResponse,
    UpdateFileResponse,
    UpdateTextResponse,
    DeleteContentResponse,
    CreateEditFormResponse,
)


__all__ = (
    "ContentFileNode",
    "AddTextNode",
    "ContentTextNode",
    "ContentCode",
    "ContentFileEditNode",
    "ContentDeleteAll",
    "DeliveryTypes",
    "ContentForm",
    "AddTextRequest",
    "EditTextRequest",
    "CreateEditFormRequest",
    "AddFileResponse",
    "AddTextResponse",
    "UpdateCodeResponse",
    "UpdateFileResponse",
    "UpdateTextResponse",
    "DeleteContentResponse",
    "CreateEditFormResponse",
)



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct

from .platform_schema import PlatformResponse
from .product_platform_name_schema import ProductPlatformNameResponse
from .product_platform_description_schema import ProductPlatformDescriptionResponse
from .product_platform_category_schema import ProductPlatformCategoryResponse
from .product_platform_pricing_schema import ProductPlatformPricingResponse
from .product_platform_bonus_schema import ProductPlatformBonusResponse
from .product_platform_guarantee_schema import ProductPlatformGuaranteeResponse
from .product_platform_discount_schema import ProductPlatformDiscountResponse
from .product_platform_parameter_schema import ProductPlatformParameterResponse
from .product_platform_image_schema import ProductPlatformImageResponse
from .product_platform_video_schema import ProductPlatformVideoResponse
from .product_platform_content_schema import ProductPlatformContentResponse


class ProductPlatformCreateRequest(Struct, forbid_unknown_fields=True):
    product_id: Annotated[
        int,
        Parameter(
            title="Product ID",
            description="The ID of the product."
        )
    ]
    platform_id: Annotated[
        int,
        Parameter(
            title="Platform ID",
            description="The ID of the platform."
        )
    ]
    custom_fields: Annotated[
        dict,
        Parameter(
            title="Custom Fields",
            description="The custom fields of the product."
        )
    ]


class ProductPlatformUpdateRequest(Struct, forbid_unknown_fields=True):
    custom_fields: Annotated[
        dict,
        Parameter(
            title="Custom Fields",
            description="The custom fields of the product."
        )
    ]
    is_visible: Annotated[
        bool,
        Parameter(
            title="Is Visible",
            description="The visibility of the product."
        )
    ]


class ProductPlatformResponse(Struct):
    id: int
    product_id: int
    platform_id: int
    version: int
    is_enabled: bool
    is_visible: bool
    custom_fields: dict
    created_at: datetime
    updated_at: datetime

    platform: PlatformResponse | None = None
    names: list[ProductPlatformNameResponse] = []
    descriptions: list[ProductPlatformDescriptionResponse] = []
    categories: list[ProductPlatformCategoryResponse] = []
    parameters: list[ProductPlatformParameterResponse] = []
    images: list[ProductPlatformImageResponse] = []
    videos: list[ProductPlatformVideoResponse] = []
    price: ProductPlatformPricingResponse | None = None
    bonus: ProductPlatformBonusResponse | None = None
    discount: ProductPlatformDiscountResponse | None = None
    guarantee: ProductPlatformGuaranteeResponse | None = None
    contents: list[ProductPlatformContentResponse] = []


class ProductPlatformShortenedResponse(Struct):
    id: int
    name: str



from __future__ import annotations

from typing import TYPE_CHECKING

from advanced_alchemy.base import SlugKey, AuditColumns, CommonTableAttributes, orm_registry
from sqlalchemy import Index, UniqueConstraint, Sequence
from sqlalchemy.dialects.postgresql import VARCHAR, ENUM, TEXT, BOOLEAN, SMALLINT
from sqlalchemy.orm import Mapped, mapped_column, relationship, declared_attr, DeclarativeBase

from .enums.location import LocationType
from .enums.part_of_world import PartOfWorldType

if TYPE_CHECKING:
    from .product import Product


class SmallIntPrimaryKey:
    """BigInt Primary Key Field Mixin."""

    # noinspection PyMethodParameters
    @declared_attr
    def id(cls) -> Mapped[int]:
        """BigInt Primary key column."""
        return mapped_column(
            SMALLINT,
            Sequence(f"{cls.__tablename__}_id_seq", optional=False),  # type: ignore[attr-defined]
            primary_key=True,
        )


class SmallIntAuditBase(CommonTableAttributes, SmallIntPrimaryKey, AuditColumns, DeclarativeBase):
    """Base for declarative models with BigInt primary keys and audit columns."""

    registry = orm_registry


# https://www.artlebedev.ru/country-list/
class Region(SmallIntAuditBase):
    @declared_attr.directive
    @classmethod
    def __table_args__(cls):
        return (
            UniqueConstraint("alpha2", name="uq_region_alpha2"),
            UniqueConstraint("alpha3", name="uq_region_alpha3"),
            Index("idx_alpha2", "alpha2"),
            Index("idx_alpha3", "alpha3"),
        )

    # id: Mapped[int] = mapped_column(
    #     SMALLINT,
    #     Sequence(f"region_id_seq", optional=False),
    #     primary_key=True
    # )

    name_ru: Mapped[str] = mapped_column(
        VARCHAR(255),
        nullable=False
    )
    name_en: Mapped[str] = mapped_column(
        VARCHAR(255),
        nullable=False
    )
    alpha2: Mapped[str] = mapped_column(
        VARCHAR(2),
        nullable=False
    )
    alpha3: Mapped[str] = mapped_column(
        VARCHAR(3),
        nullable=False
    )
    iso: Mapped[int] = mapped_column(
        SMALLINT,
        nullable=False
    )

    part_of_world: Mapped[str] = mapped_column(
        ENUM(PartOfWorldType, name="part_of_world_enum"),
        nullable=True,
    )

    location: Mapped[str | None] = mapped_column(
        ENUM(LocationType, name="location_enum"),
        nullable=True,
    )

    product_bought: Mapped[list["Product"]] = relationship(
        lazy="noload",
        back_populates="region_buy",
    )

    # products_sold: Mapped[list["Product"]] = relationship(
    #     lazy="noload",
    #     back_populates="regions_sale",
    # )



from datetime import datetime
from typing import Annotated, Any

from litestar.params import Parameter
from msgspec import Struct

from stofory_sdk.catalog.models.enums import SyncTaskStatus, SyncTaskType

from .product_platform_schema import ProductPlatformResponse

from .product_platform_sync_task_schema import ProductPlatformSyncTaskResponse
from .product_platform_external_connection_schema import ProductPlatformExternalConnectionResponse


class ValidateProductPlatfromEnablingResponse(Struct):
    result: Annotated[bool, Parameter(title="Result", description="The result of the operation.")]
    data: ProductPlatformResponse
    messages: Annotated[list[str], Parameter(title="Messages", description="Potential reasons for failure.")] = []


# class ProductPlatformSyncTaskResponse(Struct):
#     id: Annotated[int, Parameter(title="ID", description="The ID of the product platform sync task.")]
#     product_platform_id: Annotated[int, Parameter(title="Product Platform ID", description="The ID of the product platform.")]
#     task_type: Annotated[SyncTaskType, Parameter(title="Task Type", description="The type of the task.")]
#     task_meta: Annotated[dict, Parameter(title="Task Meta", description="The metadata of the task.")]
#     status: Annotated[SyncTaskStatus, Parameter(title="Status", description="The status of the task.")]
#     info: Annotated[str | None, Parameter(title="Info", description="Additional information about the task.")]
#
#     created_at: Annotated[datetime, Parameter(title="Created At", description="The timestamp of the creation of the task.")]
#     updated_at: Annotated[datetime, Parameter(title="Updated At", description="The timestamp of the last update of the task.")]
#
#
# class ProductPlatformExternalConnectionResponse(Struct):
#     id: Annotated[int, Parameter(title="ID", description="The ID of the product platform external connection.")]
#     product_id: Annotated[int, Parameter(title="Product ID", description="The ID of the product.")]
#     platform_id: Annotated[int, Parameter(title="Platform ID", description="The ID of the platform.")]
#     product_platform_id: Annotated[int, Parameter(title="Product Platform ID", description="The ID of the product platform.")]
#     version: Annotated[str, Parameter(title="Version", description="The version of the external connection.")]
#     external_id: Annotated[str, Parameter(title="External ID", description="The external ID of the external connection.")]
#     external_meta: Annotated[dict, Parameter(title="External Meta", description="The metadata of the external connection.")]
#
#     created_at: Annotated[datetime, Parameter(title="Created At", description="The timestamp of the creation of the external connection.")]
#     updated_at: Annotated[datetime, Parameter(title="Updated At", description="The timestamp of the last update of the external connection.")]


class CheckProductPlatformSyncResponse(Struct):
    result: Annotated[bool, Parameter(title="Result", description="The result of the operation.")]
    message: Annotated[str, Parameter(title="Message", description="Additional information about the task.")]
    external_connection: Annotated[ProductPlatformExternalConnectionResponse | None, Parameter(title="External Connection", description="The external connection of the product platform.")] = None
    sync_task: Annotated[ProductPlatformSyncTaskResponse | None, Parameter(title="Sync Task", description="The sync task of the product platform.")] = None


class SyncProductPlatformResponse(Struct):
    result: Annotated[bool, Parameter(title="Result", description="The result of the operation.")]
    message: Annotated[str, Parameter(title="Message", description="Additional information about the task.")]
    external_connection: Annotated[
        ProductPlatformExternalConnectionResponse | None, Parameter(title="External Connection",
                                                                    description="The external connection of the product platform.")] = None
    sync_task: Annotated[ProductPlatformSyncTaskResponse | None, Parameter(title="Sync Task", description="The sync task of the product platform.")] = None


class EnableProductPlatformResponse(Struct):
    result: Annotated[bool, Parameter(title="Result", description="The result of the operation.")]
    message: Annotated[str, Parameter(title="Message", description="Additional information about the task.")]
    external_connection: Annotated[
        ProductPlatformExternalConnectionResponse | None, Parameter(title="External Connection",
                                                                    description="The external connection of the product platform.")] = None
    sync_task: Annotated[ProductPlatformSyncTaskResponse | None, Parameter(title="Sync Task", description="The sync task of the product platform.")] = None



from typing import Any, Dict

from sqlalchemy_utils import LtreeType
from sqlalchemy_utils.types.ltree import Ltree
from sqlalchemy.types import TypeDecorator


class LtreeStr(TypeDecorator):
    impl = LtreeType

    cache_ok = True

    def process_result_value(self, value: Any, dialect: Any) -> str:
        if value is None:
            return ""
        return str(value)

    def process_bind_param(self, value: Any, dialect: Any) -> Ltree:
        if value is None:
            return Ltree("")
        return Ltree(value)



import enum


class LocationType(enum.StrEnum):
    AUSTRALIA_NEW_ZEALAND = "Австралия и Новая Зеландия"
    ANTARCTICA = "Антарктика"
    EASTERN_ASIA = "Восточная Азия"
    EASTERN_AFRICA = "Восточная Африка"
    EASTERN_EUROPE = "Восточная Европа"
    TRANSCAUCASIA = "Закавказье"
    WESTERN_ASIA = "Западная Азия"
    WESTERN_AFRICA = "Западная Африка"
    WESTERN_EUROPE = "Западная Европа"
    INDIAN_OCEAN = "Индийский океан"
    CARIBBEAN = "Карибский бассейн"
    MELANESIA = "Меланезия"
    MICRONESIA = "Микронезия"
    POLYNESIA = "Полинезия"
    NORTHERN_AFRICA = "Северная Африка"
    NORTHERN_AMERICA = "Северная Америка"
    NORTHERN_EUROPE = "Северная Европа"
    CENTRAL_AFRICA = "Центральная Африка"
    CENTRAL_AMERICA = "Центральная Америка"
    SOUTHEAST_ASIA = "Юго-Восточная Азия"
    SOUTH_AMERICA = "Южная Америка"
    SOUTHERN_EUROPE = "Южная Европа"
    SOUTHERN_AFRICA = "Южная часть Африки"
    SOUTHERN_CENTRAL_ASIA = "Южная часть Центральной Азии"
    SOUTH_OCEAN = "Южный океан"



import enum
from datetime import datetime

import msgspec

from ..schemas import Locale, Currency, BaseResponse, VariantTypes


class AsyncTaskStatuses(enum.IntEnum):
    PENDING = 0
    IN_PROGRESS = 1
    FAILED = 2
    COMPLETED = 3


class ProductStatuses(enum.StrEnum):
    DISABLED = "disabled"
    ENABLED = "enabled"


class GalleryTypes(enum.StrEnum):
    IMAGE = "image"
    VIDEO = "video"


class ProductFixedTypes(enum.StrEnum):
    TEXT = "text"
    DIGISELLERCODE = "digisellercode"
    FILE = "file"


class ProductUnfixedTypes(enum.StrEnum):
    DIGISELLERCODE = "digisellercode"


class ProductName(msgspec.Struct, forbid_unknown_fields=True):
    locale: Locale
    value: str


class ProductDescription(msgspec.Struct, forbid_unknown_fields=True):
    locale: Locale
    value: str


class ProductAddInfo(msgspec.Struct, forbid_unknown_fields=True):
    locale: Locale
    value: str


class ProductFixedPrice(msgspec.Struct, forbid_unknown_fields=True):
    price: int
    currency: Currency


class ProductUnfixedPrice(msgspec.Struct, forbid_unknown_fields=True):
    price: float
    currency: Currency
    unit_quantity: int
    unit_name: str


class CategoryOwners(enum.IntEnum):
    OWN_SHOP = 0  # собственный магазин
    PLATI_MARKET = 1  # plati.market
    WMCENTER = 2  # wmcenter.net
    GGSELL = 3  # ggsel


class Platforms(enum.StrEnum):
    PLATI = 'plati'
    WMCENTER = 'wmcentre'
    GGSELL = 'ggsel'


class ProductCategory(msgspec.Struct, forbid_unknown_fields=True):
    owner: CategoryOwners
    category_id: int


class ProductBonus(msgspec.Struct, forbid_unknown_fields=True):
    enabled: bool
    percent: int


class ProductGuarantee(msgspec.Struct, forbid_unknown_fields=True):
    enabled: bool
    value: int


class ProductVerifyCode(msgspec.Struct, forbid_unknown_fields=True):
    auto_verify: bool
    verify_url: str


class ProductPreorder(msgspec.Struct, forbid_unknown_fields=True):
    enabled: bool
    delivery_date: datetime


class InstructionTypes(enum.StrEnum):
    TEXT = "text"
    URL = "url"


class InstructionLocale(msgspec.Struct, forbid_unknown_fields=True):
    locale: Locale
    value: str


class ProductInstruction(msgspec.Struct, forbid_unknown_fields=True):
    type: InstructionTypes
    locales: list[InstructionLocale]


class SalesLimits(enum.IntEnum):
    HIDDEN = -1
    ONE = 1
    TEN = 10
    FIFTY = 50
    HUNDRED = 100
    THOUSAND = 1000


class OnlineCheckoutCategory(enum.StrEnum):
    GOODS = "Goods"  # товар
    EXERCISE_GOODS = "ExciseGoods"  # подакцизный товар
    WORK = "Work"   # работа
    SERVICE = "Service"  # услуга
    GAMBLING_BET = "GamblingBet"    # ставка в азартной игре
    GAMBLING_PRICE = "GamblingPrize"    # выигрыш в азартной игре
    LOTTERY_TICKET = "LotteryTicket"    # лотерейный билет
    LOTTERY_PRIZE = "LotteryPrize"  # выигрыш в лотерею
    INTELLECTUAL_PROPERTY_GRANT = "IntellectualPropertyGrant"   # предоставление РИД
    PAYMENT = "Payment"  # платеж
    AGENCY_FEE = "AgencyFee"    # агентское вознаграждение
    COMPOSITE_SUBJECT = "CompositeSubject"  # составной предмет расчета
    OTHER_SUBJECT = "OtherSubject"  # иной предмет расчета
    PROPERTY_RIGHT = "PropertyRight"  # имущественное право
    NON_OPERATING_INCOME = "NonOperatingIncome"  # внереализационный доход
    INSURANCE_PREMIUMS = "InsurancePremiums"  # страховой сбор
    TRADE_FEE = "TradeFee"  # торговый сбор
    RESORT_FEE = "ResortFee"   # курортный сбор
    PLEDGE = "Pledge"  # залог


class OnlineCheckoutTax(enum.StrEnum):
    VAT20 = "vat20"  # - расчетная ставка 20 %
    VAT10 = "vat10"  # - расчетная ставка 10 %
    VAT110 = "vat110"  # - расчетная ставка 10 / 110
    VAT120 = "vat120"  # - расчетная ставка 20 / 120
    VAT0 = "vat0"  # - расчетная ставка 0 %
    NO_VAT = "no_vat"  # - НДС не накладывается


class LimitationTypes(enum.StrEnum):
    NONE = "None"
    MIN_AND_MAX = "MinAndMax"
    FIXED_QUANTITY = "FixedQuantity"


class ProductUnfixedLimitation(msgspec.Struct, forbid_unknown_fields=True):
    type: LimitationTypes
    only_integer: bool
    limitations: list[int | float]


class ProductUnfixedDiscount(msgspec.Struct, forbid_unknown_fields=True):
    unit_for_discount: float
    discount: int


class CategoryName(msgspec.Struct, forbid_unknown_fields=True):
    locale: Locale
    value: str | None


class PlatformCategory(msgspec.Struct, forbid_unknown_fields=True):
    id: int
    level: int
    parent_id: int
    name: list[CategoryName]
    can_add: bool
    children: list["PlatformCategory"] | None = None


class PlatformSubcategory(msgspec.Struct, forbid_unknown_fields=True):
    id: int
    title: str
    name: list[CategoryName]


class VariantPriceUpdate(msgspec.Struct, forbid_unknown_fields=True):
    variant_id: int
    rate: float
    type: VariantTypes | None = None


class ProductIdResult(msgspec.Struct, forbid_unknown_fields=True):
    product_id: int


class StatusResult(msgspec.Struct, forbid_unknown_fields=True):
    status: str


class UploadImageResult(msgspec.Struct, forbid_unknown_fields=True):
    preview_id: int
    filename: str


class AddVideoResult(msgspec.Struct, forbid_unknown_fields=True):
    preview_id: int
    url: str


class AsyncTaskError(msgspec.Struct, forbid_unknown_fields=True):
    key: str = msgspec.field(name="Key")
    value: str = msgspec.field(name="Value")


"""

The following classes are used to validate the input and output data of the API endpoints.

"""


class CreateUniquefixedRequest(msgspec.Struct, forbid_unknown_fields=True):
    content_type: ProductFixedTypes
    name: list[ProductName]
    price: ProductFixedPrice
    categories: list[ProductCategory]
    description: list[ProductDescription]
    address_required: bool

    bonus: ProductBonus | None = None
    guarantee: ProductGuarantee | None = None
    add_info: list[ProductAddInfo] | None = None
    comission_partner: int | None = None
    trial_url: str | None = None
    preorder: ProductPreorder | None = None
    verify_code: ProductVerifyCode | None = None
    present_product_id: int | None = None
    sales_limit: SalesLimits | None = None
    online_checkout_name: str | None = None
    online_checkout_category: OnlineCheckoutCategory | None = None
    online_checkout_tax: OnlineCheckoutTax | None = None


class CreateUniquefixedResponse(BaseResponse, forbid_unknown_fields=True):
    content: ProductIdResult | None


class CreateUniqueunfixedRequest(msgspec.Struct, forbid_unknown_fields=True):
    content_type: ProductFixedTypes
    name: list[ProductName]
    prices: ProductUnfixedPrice
    categories: list[ProductCategory]
    description: list[ProductDescription]
    address_required: bool
    limitations: ProductUnfixedLimitation

    bonus: ProductBonus | None = None
    guarantee: ProductGuarantee | None = None
    add_info: list[ProductAddInfo] | None = None
    comission_partner: int | None = None
    discounts: list[ProductUnfixedDiscount] | None = None
    instruction: ProductInstruction | None = None
    present_product_id: int | None = None
    sales_limit: SalesLimits | None = None
    online_checkout_name: str | None = None
    online_checkout_category: OnlineCheckoutCategory | None = None
    online_checkout_tax: OnlineCheckoutTax | None = None


class CreateUniqueunfixedResponse(BaseResponse, forbid_unknown_fields=True):
    content: ProductIdResult | None


class UpdateUniquefixedRequest(CreateUniquefixedRequest, forbid_unknown_fields=True, kw_only=True):
    enabled: bool


class UpdateUniquefixedResponse(BaseResponse, forbid_unknown_fields=True):
    content: StatusResult | None


class UpdateUniqueunfixedRequest(CreateUniqueunfixedRequest, forbid_unknown_fields=True, kw_only=True):
    enabled: bool


class UpdateUniqueunfixedResponse(BaseResponse, forbid_unknown_fields=True):
    content: StatusResult | None


class UploadImagesResponse(BaseResponse, forbid_unknown_fields=True):
    content: list[UploadImageResult] | None


class AddVideosRequest(msgspec.Struct, forbid_unknown_fields=True):
    urls: list[str]


class AddVideosResponse(BaseResponse, forbid_unknown_fields=True):
    content: list[AddVideoResult] | None


class EditGalleryRequest(msgspec.Struct, forbid_unknown_fields=True):
    enabled: bool | None = None
    index: int | None = None
    delete: bool | None = None


# # хз шо тут делать пока, крашит базовый сервис
# class EditGalleryResponse(BaseResponse, forbid_unknown_fields=True):
#     # content: StatusResult | None
#     pass


class UpdateStatusMassRequest(msgspec.Struct, forbid_unknown_fields=True):
    new_status: ProductStatuses
    products: list[int]


class UpdateStatusMassResponse(BaseResponse, forbid_unknown_fields=True):
    content: str | None


class UpdatePriceMassRequestItem(msgspec.Struct, forbid_unknown_fields=True):
    product_id: int
    price: float
    variants: list[VariantPriceUpdate] | None = None


class UpdatePriceMassRequest(msgspec.Struct, forbid_unknown_fields=True):
    items: list[UpdatePriceMassRequestItem]


class UpdatePriceMassResponse(BaseResponse, forbid_unknown_fields=True):
    content: str | None


class AddProductToCategoryResponse(BaseResponse, forbid_unknown_fields=True):
    content: StatusResult | None


class PlatformCategoriesResponse(BaseResponse, forbid_unknown_fields=True):
    content: list[PlatformCategory] | None


class PlatformSubcategoriesResponse(BaseResponse, forbid_unknown_fields=True):
    content: list[PlatformSubcategory] | None


class AsyncTaskStatusResponse(msgspec.Struct, forbid_unknown_fields=True):
    task_id: str = msgspec.field(name="TaskId")
    status: AsyncTaskStatuses = msgspec.field(name="Status")
    success_count: int = msgspec.field(name="SuccessCount")
    error_count: int = msgspec.field(name="ErrorCount")
    total_count: int = msgspec.field(name="TotalCount")
    errors_descriptions: list[AsyncTaskError] | None = msgspec.field(name="ErrorsDescriptions")



from datetime import datetime
from typing import Annotated

from litestar.params import Parameter
from msgspec import Struct

from .localization_schema import LocalizationResponse


class ParameterNameCreateRequest(Struct, forbid_unknown_fields=True):
    parameter_id: Annotated[int, Parameter(title="Parameter ID", description="The ID of the parameter.")]
    localization_id: Annotated[
        int,
        Parameter(
            title="Localization ID",
            description="The ID of the localization."
        )
    ]
    value: Annotated[
        str,
        Parameter(
            title="Value",
            description="The value of the product parameter name."
        )
    ]


class ParameterNameUpdateRequest(Struct, forbid_unknown_fields=True):
    # localization_id: Annotated[
    #     int | None,
    #     Parameter(
    #         required=False,
    #         title="Localization ID",
    #         description="The ID of the localization."
    #     )
    # ] = None
    value: Annotated[
        str,
        Parameter(
            title="Value",
            description="The value of the product parameter name."
        )
    ]


class ParameterNameResponse(Struct):
    id: int
    parameter_id: int
    localization_id: int
    version: int
    value: str
    created_at: datetime
    updated_at: datetime

    locale: LocalizationResponse



from typing import List

import msgspec


class ExternalOptionMapping(msgspec.Struct):
    external_option_id: int
    internal_option_id: int


class ExternalParameterMapping(msgspec.Struct):
    external_parameter_id: int
    internal_parameter_id: int

    options: List[ExternalOptionMapping] | None



import enum
from typing import Optional

import msgspec


class NewOrderRequest(msgspec.Struct, forbid_unknown_fields=True):
    product_id: str
    platform: Optional[str] = None


class TypeOrder(enum.StrEnum):
    Undefined = 'Undefined'
    Steam_Gift = 'Steam_Gift'
    Steam_Balance_Refill = 'Steam_Balance_Refill'


class MinimalPaymentResponse(msgspec.Struct, forbid_unknown_fields=True):
    id_goods: int
    amount: float
    type_curr: str
    date_pay: str
    email: str
    unit_goods: Optional[int] = None
    cnt_goods: Optional[float] = None
    name: str = None
    type: TypeOrder = None


class OrderInfoResponse(msgspec.Struct, forbid_unknown_fields=True):
    data: Optional[MinimalPaymentResponse]


class OrderStatusResponse(msgspec.Struct, forbid_unknown_fields=True):
    status: Optional[str]



from datetime import datetime
from typing import Annotated, Any

from sqlalchemy_utils.types.ltree import Ltree
from litestar.params import Parameter
from msgspec import Struct

from .category_name_schema import CategoryNameResponse


class CategoryCreateRequest(Struct, forbid_unknown_fields=True):
    platform_id: Annotated[
        int,
        Parameter(
            title="Platform ID",
            description="The ID of the platform."
        )
    ]
    path: Annotated[
        Any,
        Parameter(
            title="Path",
            description="The path of the category."
        )
    ]
    parent_id: Annotated[
        int | None,
        Parameter(
            title="Parent ID",
            description="The ID of the parent category."
        )
    ] = None
    external_id: Annotated[
        int | None,
        Parameter(
            title="External ID",
            description="The external ID of the category."
        )
    ] = None
    external_parent_id: Annotated[
        int | None,
        Parameter(
            title="External Parent ID",
            description="The external ID of the parent category."
        )
    ] = None
    is_owner: Annotated[
        bool,
        Parameter(
            title="Is Owner",
            description="The owner flag of the category."
        )
    ] = False
    is_subcategory: Annotated[
        bool,
        Parameter(
            title="Is Subcategory",
            description="The subcategory flag of the category."
        )
    ] = False


class CategoryUpdateRequest(Struct, forbid_unknown_fields=True):
    platform_id: Annotated[
        int,
        Parameter(
            title="Platform ID",
            description="The ID of the platform."
        )
    ]
    parent_id: Annotated[
        int | None,
        Parameter(
            title="Parent ID",
            description="The ID of the parent category."
        )
    ]
    path: Annotated[
        Any,
        Parameter(
            title="Path",
            description="The path of the category."
        )
    ]
    external_id: Annotated[
        int | None,
        Parameter(
            title="External ID",
            description="The external ID of the category."
        )
    ] = None
    external_parent_id: Annotated[
        int | None,
        Parameter(
            title="External Parent ID",
            description="The external ID of the parent category."
        )
    ] = None
    is_owner: Annotated[
        bool,
        Parameter(
            title="Is Owner",
            description="The owner flag of the category."
        )
    ] = False
    is_subcategory: Annotated[
        bool,
        Parameter(
            title="Is Subcategory",
            description="The subcategory flag of the category."
        )
    ] = False


class CategoryResponse(Struct):
    id: int
    platform_id: int
    parent_id: int | None
    path: Any
    external_id: int | None
    external_parent_id: int | None
    is_owner: bool
    is_subcategory: bool
    created_at: datetime
    updated_at: datetime

    names: list[CategoryNameResponse]

    children_count: int | None = None



from .schemas import (
    CaptchaResponse,
    PartnerRegistrationRequest,
    UniqueCodeState,
    PaymentVerificationResponse,
    BuyerInfo,
    Content,
    SaleInfoResponse,
    PartnerRegistrationResponse,
    PermissionsResponse
)

__all__ = ('CaptchaResponse', 'PartnerRegistrationRequest', 'UniqueCodeState', 'PaymentVerificationResponse', 'BuyerInfo',
           'Content', 'SaleInfoResponse', 'PartnerRegistrationResponse', 'PermissionsResponse')
